;; -*- mode: Lisp; Syntax: Common-Lisp; Package: cells; -*-
#|

    Hunchncells -- Cells, Parenscript, and HTML

Copyright (C) 2008 by Andy Chambers

This library is free software; you can redistribute it and/or
modify it under the terms of the Lisp Lesser GNU Public License
 (http://opensource.franz.com/preamble.html), known as the LLGPL.

This library is distributed  WITHOUT ANY WARRANTY; without even 
the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the Lisp Lesser GNU Public License for more details.

|#

(in-package :openair)

(progn
  (declaim (optimize (debug 3) (speed 0) (safety 1) (compilation-speed 0))))


(defmodel html (family)
  ((id :initarg :id :accessor id :initform nil)
   (class :initarg :class :accessor cls :initform nil)
   (title :initarg :title :accessor title :initform nil)
   (style :initarg :style :accessor style :initform nil)
   (content :initarg :content :accessor content :initform nil)
   (dirtyp :initarg :dirtyp :accessor dirtyp :initform nil)
   (xhtml :initarg :xhtml :accessor xhtml :initform nil)))

(defun js-escape (str)
;;  (print (list "escaping..." str))
  (with-output-to-string (s)
    (loop for char across str do
	 (case char
	   (#\' (princ "\\'" s))
	   (otherwise (princ char s))))))

(defobserver content ((self html))
;;   (format t "~%new : ~a~%old : ~a~%"
;; 	  new-value old-value)
  (when (not (equal (apply 'concatenate 'string new-value)
		    (apply 'concatenate 'string old-value)))
    (princ (^xhtml))
    (let ((update (format nil "$(\"#~a\").html(\"~a\")" 
			  (^id)
			  (js-escape (^xhtml)))))
      (when (slot-exists-p (fm-top self) 'updates)
	(print (list "updating..." update))
	(push update (updates (fm-top self)))))))


;;; Grouping
(defhtml div () ())
(defhtml span () ())

;;; Headers
(defhtml h1 () ())
(defhtml h2 () ())
(defhtml h3 () ())

;;; Text

(defmodel text (html)
  ()
  (:default-initargs
      :xhtml (c? (with-output-to-string (s)
		   (with-html-output (s nil :prologue nil)
		     (:span :id (^id)
			    (str (^value))))))))

(defmacro mk-text (text)
  `(make-instance
    'text
    :fm-parent *parent*
    :id (gensym "S")
    :value ,text))


(defmodel page (html)
  ()
  (:default-initargs
      :xhtml (c? (with-output-to-string (s)
		   (with-html-output (s nil :prologue t :indent t)
		     (:html
		      (:head
		       (:title (str (^title)))
		       (:script :type "text/javascript"
				:src "/js/jquery.js")
		       (:script :text "text/javascript"
				:src "/js/openair.js")
		       (:style :type "text/css"
			       :href (^style)))
		      (:body
		       (:div :id "banner"
			     (:h1 (str (^title))))
		       (:div :id "content"
			     (str (apply #'concatenate 'string
					 (mapcar 'xhtml (^kids))))))))))))


;; (defun test-mk ()
;;   (xhtml
;;    (mk-page ()
;;      (mk-div ()))))

;; (xhtml )
	 
;; 	   (mk-div (:id "aa"))))
